<?php

namespace Restaurant\Feedback\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Restaurant\Feedback\Http\Requests\FeedBackRequest;
use Restaurant\Feedback\Interfaces\FeedBackRepositoryInterface;
use Restaurant\Feedback\Models\FeedBack;

/**
 * Resource controller class for feed_back.
 */
class FeedBackResourceController extends BaseController
{

    /**
     * Initialize feed_back resource controller.
     *
     * @param type FeedBackRepositoryInterface $feed_back
     *
     * @return null
     */
    public function __construct(FeedBackRepositoryInterface $feed_back)
    {
        parent::__construct();
        $this->repository = $feed_back;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Restaurant\Feedback\Repositories\Criteria\FeedBackResourceCriteria::class);
    }

    /**
     * Display a list of feed_back.
     *
     * @return Response
     */
    public function index(FeedBackRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Restaurant\Feedback\Repositories\Presenter\FeedBackPresenter::class)
                ->$function();
        }

        $feed_backs = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('feedback::feed_back.names'))
            ->view('feedback::feed_back.index', true)
            ->data(compact('feed_backs', 'view'))
            ->output();
    }

    /**
     * Display feed_back.
     *
     * @param Request $request
     * @param Model   $feed_back
     *
     * @return Response
     */
    public function show(FeedBackRequest $request, FeedBack $feed_back)
    {

        if ($feed_back->exists) {
            $view = 'feedback::feed_back.show';
        } else {
            $view = 'feedback::feed_back.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('feedback::feed_back.name'))
            ->data(compact('feed_back'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new feed_back.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FeedBackRequest $request)
    {

        $feed_back = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('feedback::feed_back.name')) 
            ->view('feedback::feed_back.create', true) 
            ->data(compact('feed_back'))
            ->output();
    }

    /**
     * Create new feed_back.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FeedBackRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $feed_back                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('feedback::feed_back.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('feedback/feed_back/' . $feed_back->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/feedback/feed_back'))
                ->redirect();
        }

    }

    /**
     * Show feed_back for editing.
     *
     * @param Request $request
     * @param Model   $feed_back
     *
     * @return Response
     */
    public function edit(FeedBackRequest $request, FeedBack $feed_back)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('feedback::feed_back.name'))
            ->view('feedback::feed_back.edit', true)
            ->data(compact('feed_back'))
            ->output();
    }

    /**
     * Update the feed_back.
     *
     * @param Request $request
     * @param Model   $feed_back
     *
     * @return Response
     */
    public function update(FeedBackRequest $request, FeedBack $feed_back)
    {
        try {
            $attributes = $request->all();

            $feed_back->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('feedback::feed_back.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('feedback/feed_back/' . $feed_back->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('feedback/feed_back/' . $feed_back->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the feed_back.
     *
     * @param Model   $feed_back
     *
     * @return Response
     */
    public function destroy(FeedBackRequest $request, FeedBack $feed_back)
    {
        try {

            $feed_back->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('feedback::feed_back.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('feedback/feed_back/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('feedback/feed_back/' . $feed_back->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple feed_back.
     *
     * @param Model   $feed_back
     *
     * @return Response
     */
    public function delete(FeedBackRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('feedback::feed_back.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('feedback/feed_back'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/feedback/feed_back'))
                ->redirect();
        }

    }

    /**
     * Restore deleted feed_backs.
     *
     * @param Model   $feed_back
     *
     * @return Response
     */
    public function restore(FeedBackRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('feedback::feed_back.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/feedback/feed_back'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/feedback/feed_back/'))
                ->redirect();
        }

    }

}
